@@ -60,6 +60,7 @@ import org.eclipse.jgit.patch.FileHeader;
 import org.eclipse.jgit.patch.HunkHeader;
 import org.eclipse.jgit.treewalk.FileTreeIterator;
 import org.eclipse.jgit.treewalk.filter.PathFilter;
+import org.eclipse.jgit.util.FileUtils;
 import org.eclipse.jgit.util.RawParseUtils;
 import org.eclipse.jgit.util.io.DisabledOutputStream;
 import org.eclipse.jgit.util.io.SafeBufferedOutputStream;
@@ -285,11 +286,34 @@ public class DiffFormatterTest extends RepositoryTestCase {
 		assertEquals(expected, fh.getScriptText());
 	}
 
+	@Test
+	public void testCreateFileHeaderForRenameModeChange()
+			throws Exception {
+		DiffEntry a = DiffEntry.delete(PATH_A, ObjectId.zeroId());
+		DiffEntry b = DiffEntry.add(PATH_B, ObjectId.zeroId());
+		b.oldMode = FileMode.REGULAR_FILE;
+		b.newMode = FileMode.EXECUTABLE_FILE;
+		DiffEntry m = DiffEntry.pair(ChangeType.RENAME, a, b, 100);
+		m.oldId = null;
+		m.newId = null;
+
+		FileHeader fh = df.toFileHeader(m);
+		//@formatter:off
+		String expected = DIFF + "a/src/a b/src/b\n" +
+				"old mode 100644\n" +
+				"new mode 100755\n" +
+				"similarity index 100%\n" +
+				"rename from src/a\n" +
+				"rename to src/b\n";
+		//@formatter:on
+		assertEquals(expected, fh.getScriptText());
+	}
+
 	@Test
 	public void testDiff() throws Exception {
 		write(new File(db.getDirectory().getParent(), "test.txt"), "test");
 		File folder = new File(db.getDirectory().getParent(), "folder");
-		folder.mkdir();
+		FileUtils.mkdir(folder);
 		write(new File(folder, "folder.txt"), "folder");
 		Git git = new Git(db);
 		git.add().addFilepattern(".").call();
@@ -305,7 +329,7 @@ public class DiffFormatterTest extends RepositoryTestCase {
 		df.format(oldTree, newTree);
 		df.flush();
 
-		String actual = os.toString();
+		String actual = os.toString("UTF-8");
 		String expected =
  "diff --git a/folder/folder.txt b/folder/folder.txt\n"
 				+ "index 0119635..95c4c65 100644\n"
@@ -314,7 +338,7 @@ public class DiffFormatterTest extends RepositoryTestCase {
 				+ "\\ No newline at end of file\n" + "+folder change\n"
 				+ "\\ No newline at end of file\n";
 
-		assertEquals(expected.toString(), actual);
+		assertEquals(expected, actual);
 	}
 
 	private static String makeDiffHeader(String pathA, String pathB,
