@@ -66,14 +66,22 @@ import java.util.Set;
 import org.eclipse.jgit.api.MergeCommand.FastForwardMode;
 import org.eclipse.jgit.errors.ConfigInvalidException;
 import org.eclipse.jgit.junit.MockSystemReader;
+import org.eclipse.jgit.merge.MergeConfig;
 import org.eclipse.jgit.util.FS;
 import org.eclipse.jgit.util.SystemReader;
+import org.junit.After;
 import org.junit.Test;
 
 /**
  * Test reading of git config
  */
 public class ConfigTest {
+
+	@After
+	public void tearDown() {
+		SystemReader.setInstance(null);
+	}
+
 	@Test
 	public void test001_ReadBareKey() throws ConfigInvalidException {
 		final Config c = parse("[foo]\nbar\n");
@@ -327,19 +335,27 @@ public class ConfigTest {
 		assertSame(FastForwardMode.FF, c.getEnum(
 				ConfigConstants.CONFIG_BRANCH_SECTION, "side",
 				ConfigConstants.CONFIG_KEY_MERGEOPTIONS, FastForwardMode.FF));
+		MergeConfig mergeConfig = c.get(MergeConfig.getParser("side"));
+		assertSame(FastForwardMode.FF, mergeConfig.getFastForwardMode());
 		c = parse("[branch \"side\"]\n\tmergeoptions = --ff-only\n");
 		assertSame(FastForwardMode.FF_ONLY, c.getEnum(
 				ConfigConstants.CONFIG_BRANCH_SECTION, "side",
 				ConfigConstants.CONFIG_KEY_MERGEOPTIONS,
 				FastForwardMode.FF_ONLY));
+		mergeConfig = c.get(MergeConfig.getParser("side"));
+		assertSame(FastForwardMode.FF_ONLY, mergeConfig.getFastForwardMode());
 		c = parse("[branch \"side\"]\n\tmergeoptions = --ff\n");
 		assertSame(FastForwardMode.FF, c.getEnum(
 				ConfigConstants.CONFIG_BRANCH_SECTION, "side",
 				ConfigConstants.CONFIG_KEY_MERGEOPTIONS, FastForwardMode.FF));
+		mergeConfig = c.get(MergeConfig.getParser("side"));
+		assertSame(FastForwardMode.FF, mergeConfig.getFastForwardMode());
 		c = parse("[branch \"side\"]\n\tmergeoptions = --no-ff\n");
 		assertSame(FastForwardMode.NO_FF, c.getEnum(
 				ConfigConstants.CONFIG_BRANCH_SECTION, "side",
 				ConfigConstants.CONFIG_KEY_MERGEOPTIONS, FastForwardMode.NO_FF));
+		mergeConfig = c.get(MergeConfig.getParser("side"));
+		assertSame(FastForwardMode.NO_FF, mergeConfig.getFastForwardMode());
 	}
 
 	@Test
@@ -361,18 +377,56 @@ public class ConfigTest {
 		assertSame(FastForwardMode.Merge.TRUE, c.getEnum(
 				ConfigConstants.CONFIG_KEY_MERGE, null,
 				ConfigConstants.CONFIG_KEY_FF, FastForwardMode.Merge.TRUE));
+		MergeConfig mergeConfig = c.get(MergeConfig.getParser("side"));
+		assertSame(FastForwardMode.FF, mergeConfig.getFastForwardMode());
 		c = parse("[merge]\n\tff = only\n");
 		assertSame(FastForwardMode.Merge.ONLY, c.getEnum(
 				ConfigConstants.CONFIG_KEY_MERGE, null,
 				ConfigConstants.CONFIG_KEY_FF, FastForwardMode.Merge.ONLY));
+		mergeConfig = c.get(MergeConfig.getParser("side"));
+		assertSame(FastForwardMode.FF_ONLY, mergeConfig.getFastForwardMode());
 		c = parse("[merge]\n\tff = true\n");
 		assertSame(FastForwardMode.Merge.TRUE, c.getEnum(
 				ConfigConstants.CONFIG_KEY_MERGE, null,
 				ConfigConstants.CONFIG_KEY_FF, FastForwardMode.Merge.TRUE));
+		mergeConfig = c.get(MergeConfig.getParser("side"));
+		assertSame(FastForwardMode.FF, mergeConfig.getFastForwardMode());
 		c = parse("[merge]\n\tff = false\n");
 		assertSame(FastForwardMode.Merge.FALSE, c.getEnum(
 				ConfigConstants.CONFIG_KEY_MERGE, null,
 				ConfigConstants.CONFIG_KEY_FF, FastForwardMode.Merge.FALSE));
+		mergeConfig = c.get(MergeConfig.getParser("side"));
+		assertSame(FastForwardMode.NO_FF, mergeConfig.getFastForwardMode());
+	}
+
+	@Test
+	public void testCombinedMergeOptions() throws ConfigInvalidException {
+		Config c = new Config(null); // not set
+		MergeConfig mergeConfig = c.get(MergeConfig.getParser("side"));
+		assertSame(FastForwardMode.FF, mergeConfig.getFastForwardMode());
+		assertTrue(mergeConfig.isCommit());
+		assertFalse(mergeConfig.isSquash());
+		// branch..mergeoptions should win over merge.ff
+		c = parse("[merge]\n\tff = false\n"
+				+ "[branch \"side\"]\n\tmergeoptions = --ff-only\n");
+		mergeConfig = c.get(MergeConfig.getParser("side"));
+		assertSame(FastForwardMode.FF_ONLY, mergeConfig.getFastForwardMode());
+		assertTrue(mergeConfig.isCommit());
+		assertFalse(mergeConfig.isSquash());
+		// merge.ff used for ff setting if not set via mergeoptions
+		c = parse("[merge]\n\tff = only\n"
+				+ "[branch \"side\"]\n\tmergeoptions = --squash\n");
+		mergeConfig = c.get(MergeConfig.getParser("side"));
+		assertSame(FastForwardMode.FF_ONLY, mergeConfig.getFastForwardMode());
+		assertTrue(mergeConfig.isCommit());
+		assertTrue(mergeConfig.isSquash());
+		// mergeoptions wins if it has ff options amongst other options
+		c = parse("[merge]\n\tff = false\n"
+				+ "[branch \"side\"]\n\tmergeoptions = --ff-only --no-commit\n");
+		mergeConfig = c.get(MergeConfig.getParser("side"));
+		assertSame(FastForwardMode.FF_ONLY, mergeConfig.getFastForwardMode());
+		assertFalse(mergeConfig.isCommit());
+		assertFalse(mergeConfig.isSquash());
 	}
 
 	@Test
@@ -512,6 +566,31 @@ public class ConfigTest {
 		assertFalse(itr.hasNext());
 	}
 
+	@Test
+	public void test_ReadNamesInSectionRecursive()
+			throws ConfigInvalidException {
+		String baseConfigString = "[core]\n" + "logAllRefUpdates = true\n";
+		String configString = "[core]\n" + "repositoryFormatVersion = 0\n"
+				+ "filemode = false\n";
+		final Config c = parse(configString, parse(baseConfigString));
+		Set<String> names = c.getNames("core", true);
+		assertEquals("Core section size", 3, names.size());
+		assertTrue("Core section should contain \"filemode\"",
+				names.contains("filemode"));
+		assertTrue("Core section should contain \"repositoryFormatVersion\"",
+				names.contains("repositoryFormatVersion"));
+		assertTrue("Core section should contain \"logAllRefUpdates\"",
+				names.contains("logAllRefUpdates"));
+		assertTrue("Core section should contain \"logallrefupdates\"",
+				names.contains("logallrefupdates"));
+
+		Iterator<String> itr = names.iterator();
+		assertEquals("filemode", itr.next());
+		assertEquals("repositoryFormatVersion", itr.next());
+		assertEquals("logAllRefUpdates", itr.next());
+		assertFalse(itr.hasNext());
+	}
+
 	@Test
 	public void test010_readNamesInSubSection() throws ConfigInvalidException {
 		String configString = "[a \"sub1\"]\n"//
@@ -533,6 +612,30 @@ public class ConfigTest {
 		assertTrue("Subsection should contain \"b\"", names.contains("b"));
 	}
 
+	@Test
+	public void readNamesInSubSectionRecursive() throws ConfigInvalidException {
+		String baseConfigString = "[a \"sub1\"]\n"//
+				+ "x = 0\n" //
+				+ "y = false\n"//
+				+ "[a \"sub2\"]\n"//
+				+ "A=0\n";//
+		String configString = "[a \"sub1\"]\n"//
+				+ "z = true\n"//
+				+ "[a \"sub2\"]\n"//
+				+ "B=1\n";
+		final Config c = parse(configString, parse(baseConfigString));
+		Set<String> names = c.getNames("a", "sub1", true);
+		assertEquals("Subsection size", 3, names.size());
+		assertTrue("Subsection should contain \"x\"", names.contains("x"));
+		assertTrue("Subsection should contain \"y\"", names.contains("y"));
+		assertTrue("Subsection should contain \"z\"", names.contains("z"));
+		names = c.getNames("a", "sub2", true);
+		assertEquals("Subsection size", 2, names.size());
+		assertTrue("Subsection should contain \"A\"", names.contains("A"));
+		assertTrue("Subsection should contain \"a\"", names.contains("a"));
+		assertTrue("Subsection should contain \"B\"", names.contains("B"));
+	}
+
 	@Test
 	public void testQuotingForSubSectionNames() {
 		String resultPattern = "[testsection \"{0}\"]\n\ttestname = testvalue\n";
@@ -577,7 +680,12 @@ public class ConfigTest {
 
 	private static Config parse(final String content)
 			throws ConfigInvalidException {
-		final Config c = new Config(null);
+		return parse(content, null);
+	}
+
+	private static Config parse(final String content, Config baseConfig)
+			throws ConfigInvalidException {
+		final Config c = new Config(baseConfig);
 		c.fromText(content);
 		return c;
 	}
