@@ -56,13 +56,14 @@ import org.eclipse.jgit.api.Git;
 import org.eclipse.jgit.dircache.DirCache;
 import org.eclipse.jgit.dircache.DirCacheCheckout;
 import org.eclipse.jgit.dircache.DirCacheEditor;
+import org.eclipse.jgit.dircache.DirCacheEditor.PathEdit;
 import org.eclipse.jgit.dircache.DirCacheEntry;
 import org.eclipse.jgit.dircache.DirCacheIterator;
-import org.eclipse.jgit.dircache.DirCacheEditor.PathEdit;
 import org.eclipse.jgit.errors.CorruptObjectException;
 import org.eclipse.jgit.errors.IncorrectObjectTypeException;
 import org.eclipse.jgit.errors.MissingObjectException;
 import org.eclipse.jgit.junit.RepositoryTestCase;
+import org.eclipse.jgit.lib.ConfigConstants;
 import org.eclipse.jgit.lib.Constants;
 import org.eclipse.jgit.lib.FileMode;
 import org.eclipse.jgit.lib.ObjectId;
@@ -98,6 +99,19 @@ public class FileTreeIteratorTest extends RepositoryTestCase {
 		}
 	}
 
+	@Test
+	public void testGetEntryContentLength() throws Exception {
+		final FileTreeIterator fti = new FileTreeIterator(db);
+		fti.next(1);
+		assertEquals(3, fti.getEntryContentLength());
+		fti.back(1);
+		assertEquals(2, fti.getEntryContentLength());
+		fti.next(1);
+		assertEquals(3, fti.getEntryContentLength());
+		fti.reset();
+		assertEquals(2, fti.getEntryContentLength());
+	}
+
 	@Test
 	public void testEmptyIfRootIsFile() throws Exception {
 		final File r = new File(trash, paths[0]);
@@ -223,13 +237,17 @@ public class FileTreeIteratorTest extends RepositoryTestCase {
 		ObjectId fromRaw = ObjectId.fromRaw(fti.idBuffer(), fti.idOffset());
 		assertEquals("6b584e8ece562ebffc15d38808cd6b98fc3d97ea",
 				fromRaw.getName());
-		assertFalse(fti.isModified(dce, false));
+		ObjectReader objectReader = db.newObjectReader();
+		assertFalse(fti.isModified(dce, false, objectReader));
+		objectReader.release();
 	}
 
 	@Test
-	public void testIsModifiedSymlink() throws Exception {
+	public void testIsModifiedSymlinkAsFile() throws Exception {
 		File f = writeTrashFile("symlink", "content");
 		Git git = new Git(db);
+		db.getConfig().setString(ConfigConstants.CONFIG_CORE_SECTION, null,
+				ConfigConstants.CONFIG_KEY_SYMLINKS, "false");
 		git.add().addFilepattern("symlink").call();
 		git.commit().setMessage("commit").call();
 
@@ -242,7 +260,9 @@ public class FileTreeIteratorTest extends RepositoryTestCase {
 				.getConfig().get(WorkingTreeOptions.KEY));
 		while (!fti.getEntryPathString().equals("symlink"))
 			fti.next(1);
-		assertFalse(fti.isModified(dce, false));
+		ObjectReader objectReader = db.newObjectReader();
+		assertFalse(fti.isModified(dce, false, objectReader));
+		objectReader.release();
 	}
 
 	@Test
@@ -265,7 +285,9 @@ public class FileTreeIteratorTest extends RepositoryTestCase {
 		// If the rounding trick does not work we could skip the compareMetaData
 		// test and hope that we are usually testing the intended code path.
 		assertEquals(MetadataDiff.SMUDGED, fti.compareMetadata(dce));
-		assertTrue(fti.isModified(dce, false));
+		ObjectReader objectReader = db.newObjectReader();
+		assertTrue(fti.isModified(dce, false, objectReader));
+		objectReader.release();
 	}
 
 	@Test
