@@ -124,6 +124,26 @@ public class Git {
 		return new Git(repo);
 	}
 
+	/**
+	 * Frees resources held by the underlying {@link Repository} instance. It is
+	 * recommended to call this method as soon as you don't need a reference to
+	 * this {@link Git} instance and the underlying {@link Repository} instance
+	 * anymore. This method closes the underlying object and ref databases. This
+	 * will free memory and file handles. E.g. on Windows the repository will
+	 * keep file handles on pack files unless you call this method. Such open
+	 * file handles may for example prevent that the repository folder in the
+	 * filesystem can be deleted.
+	 * <p>
+	 * After calling close() you should not use this {@link Git} instance and
+	 * the underlying {@link Repository} instance anymore.
+	 *
+	 * @since 3.2
+	 */
+	public void close() {
+		if (repo != null)
+			repo.close();
+	}
+
 	/**
 	 * Returns a command object to execute a {@code clone} command
 	 *
@@ -663,6 +683,17 @@ public class Git {
 		return new NameRevCommand(repo);
 	}
 
+	/**
+	 * Returns a command object to come up with a short name that describes a
+	 * commit in terms of the nearest git tag.
+	 *
+	 * @return a {@link DescribeCommand}.
+	 * @since 3.2
+	 */
+	public DescribeCommand describe() {
+		return new DescribeCommand(repo);
+	}
+
 	/**
 	 * @return the git repository this class is interacting with
 	 */
