@@ -42,32 +42,35 @@
  */
 package org.eclipse.jgit.api;
 
-import java.io.BufferedReader;
-import java.io.BufferedWriter;
 import java.io.ByteArrayOutputStream;
 import java.io.File;
-import java.io.FileInputStream;
 import java.io.FileNotFoundException;
 import java.io.FileOutputStream;
 import java.io.IOException;
-import java.io.InputStreamReader;
-import java.io.OutputStreamWriter;
 import java.text.MessageFormat;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
+import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
 
 import org.eclipse.jgit.api.RebaseResult.Status;
+import org.eclipse.jgit.api.ResetCommand.ResetType;
 import org.eclipse.jgit.api.errors.CheckoutConflictException;
+import org.eclipse.jgit.api.errors.ConcurrentRefUpdateException;
 import org.eclipse.jgit.api.errors.GitAPIException;
+import org.eclipse.jgit.api.errors.InvalidRebaseStepException;
 import org.eclipse.jgit.api.errors.InvalidRefNameException;
 import org.eclipse.jgit.api.errors.JGitInternalException;
 import org.eclipse.jgit.api.errors.NoHeadException;
+import org.eclipse.jgit.api.errors.NoMessageException;
 import org.eclipse.jgit.api.errors.RefAlreadyExistsException;
 import org.eclipse.jgit.api.errors.RefNotFoundException;
+import org.eclipse.jgit.api.errors.StashApplyFailureException;
 import org.eclipse.jgit.api.errors.UnmergedPathsException;
 import org.eclipse.jgit.api.errors.WrongRepositoryStateException;
 import org.eclipse.jgit.diff.DiffFormatter;
@@ -77,12 +80,15 @@ import org.eclipse.jgit.dircache.DirCacheIterator;
 import org.eclipse.jgit.internal.JGitText;
 import org.eclipse.jgit.lib.AbbreviatedObjectId;
 import org.eclipse.jgit.lib.AnyObjectId;
+import org.eclipse.jgit.lib.ConfigConstants;
 import org.eclipse.jgit.lib.Constants;
 import org.eclipse.jgit.lib.NullProgressMonitor;
 import org.eclipse.jgit.lib.ObjectId;
 import org.eclipse.jgit.lib.ObjectReader;
 import org.eclipse.jgit.lib.PersonIdent;
 import org.eclipse.jgit.lib.ProgressMonitor;
+import org.eclipse.jgit.lib.RebaseTodoLine;
+import org.eclipse.jgit.lib.RebaseTodoLine.Action;
 import org.eclipse.jgit.lib.Ref;
 import org.eclipse.jgit.lib.RefUpdate;
 import org.eclipse.jgit.lib.RefUpdate.Result;
@@ -150,6 +156,14 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 
 	private static final String AMEND = "amend"; //$NON-NLS-1$
 
+	private static final String MESSAGE_FIXUP = "message-fixup"; //$NON-NLS-1$
+
+	private static final String MESSAGE_SQUASH = "message-squash"; //$NON-NLS-1$
+
+	private static final String AUTOSTASH = "autostash"; //$NON-NLS-1$
+
+	private static final String AUTOSTASH_MSG = "On {0}: autostash";
+
 	/**
 	 * The available operations
 	 */
@@ -169,7 +183,12 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 		/**
 		 * Aborts and resets the current rebase
 		 */
-		ABORT;
+		ABORT,
+		/**
+		 * Starts processing steps
+		 * @since 3.2
+		 */
+		PROCESS_STEPS;
 	}
 
 	private Operation operation = Operation.BEGIN;
@@ -186,6 +205,12 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 
 	private InteractiveHandler interactiveHandler;
 
+	private boolean stopAfterInitialization = false;
+
+	private RevCommit newHead;
+
+	private boolean lastStepWasForward;
+
 	/**
 	 * @param repo
 	 */
@@ -209,8 +234,8 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 	 */
 	public RebaseResult call() throws GitAPIException, NoHeadException,
 			RefNotFoundException, WrongRepositoryStateException {
-		RevCommit newHead = null;
-		boolean lastStepWasForward = false;
+		newHead = null;
+		lastStepWasForward = false;
 		checkCallable();
 		checkParameters();
 		try {
@@ -221,6 +246,8 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 				} catch (IOException ioe) {
 					throw new JGitInternalException(ioe.getMessage(), ioe);
 				}
+			case PROCESS_STEPS:
+				// fall through
 			case SKIP:
 				// fall through
 			case CONTINUE:
@@ -236,9 +263,29 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 						.resolve(upstreamCommitId));
 				break;
 			case BEGIN:
+				autoStash();
+				if (stopAfterInitialization
+						|| !walk.isMergedInto(
+								walk.parseCommit(repo.resolve(Constants.HEAD)),
+								upstreamCommit)) {
+					org.eclipse.jgit.api.Status status = Git.wrap(repo)
+							.status().call();
+					if (status.hasUncommittedChanges()) {
+						List<String> list = new ArrayList<String>();
+						list.addAll(status.getUncommittedChanges());
+						return RebaseResult.uncommittedChanges(list);
+					}
+				}
 				RebaseResult res = initFilesAndRewind();
-				if (res != null)
+				if (stopAfterInitialization)
+					return RebaseResult.INTERACTIVE_PREPARED_RESULT;
+				if (res != null) {
+					autoStashApply();
+					if (rebaseState.getDir().exists())
+						FileUtils.delete(rebaseState.getDir(),
+								FileUtils.RECURSIVE);
 					return res;
+				}
 			}
 
 			if (monitor.isCancelled())
@@ -246,7 +293,19 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 
 			if (operation == Operation.CONTINUE) {
 				newHead = continueRebase();
-
+				List<RebaseTodoLine> doneLines = repo.readRebaseTodo(
+						rebaseState.getPath(DONE), true);
+				RebaseTodoLine step = doneLines.get(doneLines.size() - 1);
+				if (newHead != null
+						&& step.getAction() != Action.PICK) {
+					RebaseTodoLine newStep = new RebaseTodoLine(
+							step.getAction(),
+							AbbreviatedObjectId.fromObjectId(newHead),
+							step.getShortMessage());
+					RebaseResult result = processStep(newStep, false);
+					if (result != null)
+						return result;
+				}
 				File amendFile = rebaseState.getFile(AMEND);
 				boolean amendExists = amendFile.exists();
 				if (amendExists) {
@@ -265,109 +324,358 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 			if (operation == Operation.SKIP)
 				newHead = checkoutCurrentHead();
 
-			ObjectReader or = repo.newObjectReader();
-
-			List<Step> steps = loadSteps();
+			List<RebaseTodoLine> steps = repo.readRebaseTodo(
+					rebaseState.getPath(GIT_REBASE_TODO), false);
+			if (steps.size() == 0) {
+				return finishRebase(walk.parseCommit(repo.resolve(Constants.HEAD)), false);
+			}
 			if (isInteractive()) {
 				interactiveHandler.prepareSteps(steps);
-				BufferedWriter fw = new BufferedWriter(new OutputStreamWriter(
-						new FileOutputStream(
-								rebaseState.getFile(GIT_REBASE_TODO)),
-								Constants.CHARACTER_ENCODING));
-				fw.newLine();
-				try {
-					StringBuilder sb = new StringBuilder();
-					for (Step step : steps) {
-						sb.setLength(0);
-						sb.append(step.action.token);
-						sb.append(" "); //$NON-NLS-1$
-						sb.append(step.commit.name());
-						sb.append(" "); //$NON-NLS-1$
-						sb.append(RawParseUtils.decode(step.shortMessage)
-								.trim());
-						fw.write(sb.toString());
-						fw.newLine();
-					}
-				} finally {
-					fw.close();
-				}
+				repo.writeRebaseTodoFile(rebaseState.getPath(GIT_REBASE_TODO),
+						steps, false);
 			}
-			for (Step step : steps) {
+			checkSteps(steps);
+			for (int i = 0; i < steps.size(); i++) {
+				RebaseTodoLine step = steps.get(i);
 				popSteps(1);
-				Collection<ObjectId> ids = or.resolve(step.commit);
-				if (ids.size() != 1)
-					throw new JGitInternalException(
-							"Could not resolve uniquely the abbreviated object ID");
-				RevCommit commitToPick = walk
-						.parseCommit(ids.iterator().next());
-				if (monitor.isCancelled())
-					return new RebaseResult(commitToPick);
-				try {
-					monitor.beginTask(MessageFormat.format(
-							JGitText.get().applyingCommit,
-							commitToPick.getShortMessage()),
-							ProgressMonitor.UNKNOWN);
-					// if the first parent of commitToPick is the current HEAD,
-					// we do a fast-forward instead of cherry-pick to avoid
-					// unnecessary object rewriting
-					newHead = tryFastForward(commitToPick);
-					lastStepWasForward = newHead != null;
-					if (!lastStepWasForward) {
-						// TODO if the content of this commit is already merged
-						// here we should skip this step in order to avoid
-						// confusing pseudo-changed
-						String ourCommitName = getOurCommitName();
-						CherryPickResult cherryPickResult = new Git(repo)
-								.cherryPick().include(commitToPick)
-								.setOurCommitName(ourCommitName)
-								.setReflogPrefix("rebase:").call(); //$NON-NLS-1$
-						switch (cherryPickResult.getStatus()) {
-						case FAILED:
-							if (operation == Operation.BEGIN)
-								return abort(new RebaseResult(
-										cherryPickResult.getFailingPaths()));
-							else
-								return stop(commitToPick);
-						case CONFLICTING:
-							return stop(commitToPick);
-						case OK:
-							newHead = cherryPickResult.getNewHead();
-						}
-					}
-					switch (step.action) {
-					case PICK:
-						continue; // continue rebase process on pick command
-					case REWORD:
-						String oldMessage = commitToPick.getFullMessage();
-						String newMessage = interactiveHandler
-								.modifyCommitMessage(oldMessage);
-						newHead = new Git(repo).commit().setMessage(newMessage)
-								.setAmend(true).call();
-						continue;
-					case EDIT:
-						rebaseState.createFile(AMEND, commitToPick.name());
-						return stop(commitToPick);
-					}
-				} finally {
-					monitor.endTask();
+				RebaseResult result = processStep(step, true);
+				if (result != null) {
+					return result;
 				}
 			}
-			if (newHead != null) {
-				String headName = rebaseState.readFile(HEAD_NAME);
-				updateHead(headName, newHead, upstreamCommit);
-				FileUtils.delete(rebaseState.getDir(), FileUtils.RECURSIVE);
-				if (lastStepWasForward)
-					return RebaseResult.FAST_FORWARD_RESULT;
-				return RebaseResult.OK_RESULT;
-			}
-			return RebaseResult.FAST_FORWARD_RESULT;
+			return finishRebase(newHead, lastStepWasForward);
 		} catch (CheckoutConflictException cce) {
-			return new RebaseResult(cce.getConflictingPaths());
+			return RebaseResult.conflicts(cce.getConflictingPaths());
 		} catch (IOException ioe) {
 			throw new JGitInternalException(ioe.getMessage(), ioe);
 		}
 	}
 
+	private void autoStash() throws GitAPIException, IOException {
+		if (repo.getConfig().getBoolean(ConfigConstants.CONFIG_REBASE_SECTION,
+				ConfigConstants.CONFIG_KEY_AUTOSTASH, false)) {
+			String message = MessageFormat.format(
+							AUTOSTASH_MSG,
+							Repository
+									.shortenRefName(getHeadName(getHead())));
+			RevCommit stashCommit = Git.wrap(repo).stashCreate().setRef(null)
+					.setWorkingDirectoryMessage(
+							message)
+					.call();
+			if (stashCommit != null) {
+				FileUtils.mkdir(rebaseState.getDir());
+				rebaseState.createFile(AUTOSTASH, stashCommit.getName());
+			}
+		}
+	}
+
+	private boolean autoStashApply() throws IOException, GitAPIException {
+		boolean conflicts = false;
+		if (rebaseState.getFile(AUTOSTASH).exists()) {
+			String stash = rebaseState.readFile(AUTOSTASH);
+			try {
+				Git.wrap(repo).stashApply().setStashRef(stash)
+						.ignoreRepositoryState(true).call();
+			} catch (StashApplyFailureException e) {
+				conflicts = true;
+				RevWalk rw = new RevWalk(repo);
+				ObjectId stashId = repo.resolve(stash);
+				RevCommit commit = rw.parseCommit(stashId);
+				updateStashRef(commit, commit.getAuthorIdent(),
+						commit.getShortMessage());
+			}
+		}
+		return conflicts;
+	}
+
+	private void updateStashRef(ObjectId commitId, PersonIdent refLogIdent,
+			String refLogMessage) throws IOException {
+		Ref currentRef = repo.getRef(Constants.R_STASH);
+		RefUpdate refUpdate = repo.updateRef(Constants.R_STASH);
+		refUpdate.setNewObjectId(commitId);
+		refUpdate.setRefLogIdent(refLogIdent);
+		refUpdate.setRefLogMessage(refLogMessage, false);
+		if (currentRef != null)
+			refUpdate.setExpectedOldObjectId(currentRef.getObjectId());
+		else
+			refUpdate.setExpectedOldObjectId(ObjectId.zeroId());
+		refUpdate.forceUpdate();
+	}
+
+	private RebaseResult processStep(RebaseTodoLine step, boolean shouldPick)
+			throws IOException, GitAPIException {
+		if (Action.COMMENT.equals(step.getAction()))
+			return null;
+		ObjectReader or = repo.newObjectReader();
+
+		Collection<ObjectId> ids = or.resolve(step.getCommit());
+		if (ids.size() != 1)
+			throw new JGitInternalException(
+					"Could not resolve uniquely the abbreviated object ID");
+		RevCommit commitToPick = walk.parseCommit(ids.iterator().next());
+		if (shouldPick) {
+			if (monitor.isCancelled())
+				return RebaseResult.result(Status.STOPPED, commitToPick);
+			RebaseResult result = cherryPickCommit(commitToPick);
+			if (result != null)
+				return result;
+		}
+		boolean isSquash = false;
+		switch (step.getAction()) {
+		case PICK:
+			return null; // continue rebase process on pick command
+		case REWORD:
+			String oldMessage = commitToPick.getFullMessage();
+			String newMessage = interactiveHandler
+					.modifyCommitMessage(oldMessage);
+			newHead = new Git(repo).commit().setMessage(newMessage)
+					.setAmend(true).call();
+			return null;
+		case EDIT:
+			rebaseState.createFile(AMEND, commitToPick.name());
+			return stop(commitToPick, Status.EDIT);
+		case COMMENT:
+			break;
+		case SQUASH:
+			isSquash = true;
+			//$FALL-THROUGH$
+		case FIXUP:
+			resetSoftToParent();
+			List<RebaseTodoLine> steps = repo.readRebaseTodo(
+					rebaseState.getPath(GIT_REBASE_TODO), false);
+			RebaseTodoLine nextStep = steps.size() > 0 ? steps.get(0) : null;
+			File messageFixupFile = rebaseState.getFile(MESSAGE_FIXUP);
+			File messageSquashFile = rebaseState.getFile(MESSAGE_SQUASH);
+			if (isSquash && messageFixupFile.exists())
+				messageFixupFile.delete();
+			newHead = doSquashFixup(isSquash, commitToPick, nextStep,
+					messageFixupFile, messageSquashFile);
+		}
+		return null;
+	}
+
+	private RebaseResult cherryPickCommit(RevCommit commitToPick)
+			throws IOException, GitAPIException, NoMessageException,
+			UnmergedPathsException, ConcurrentRefUpdateException,
+			WrongRepositoryStateException, NoHeadException {
+		try {
+			monitor.beginTask(MessageFormat.format(
+					JGitText.get().applyingCommit,
+					commitToPick.getShortMessage()), ProgressMonitor.UNKNOWN);
+			// if the first parent of commitToPick is the current HEAD,
+			// we do a fast-forward instead of cherry-pick to avoid
+			// unnecessary object rewriting
+			newHead = tryFastForward(commitToPick);
+			lastStepWasForward = newHead != null;
+			if (!lastStepWasForward) {
+				// TODO if the content of this commit is already merged
+				// here we should skip this step in order to avoid
+				// confusing pseudo-changed
+				String ourCommitName = getOurCommitName();
+				CherryPickResult cherryPickResult = new Git(repo).cherryPick()
+						.include(commitToPick).setOurCommitName(ourCommitName)
+						.setReflogPrefix("rebase:").call(); //$NON-NLS-1$
+				switch (cherryPickResult.getStatus()) {
+				case FAILED:
+					if (operation == Operation.BEGIN)
+						return abort(RebaseResult.failed(cherryPickResult
+								.getFailingPaths()));
+					else
+						return stop(commitToPick, Status.STOPPED);
+				case CONFLICTING:
+					return stop(commitToPick, Status.STOPPED);
+				case OK:
+					newHead = cherryPickResult.getNewHead();
+				}
+			}
+			return null;
+		} finally {
+			monitor.endTask();
+		}
+	}
+
+	private RebaseResult finishRebase(RevCommit newHead,
+			boolean lastStepWasForward) throws IOException, GitAPIException {
+		String headName = rebaseState.readFile(HEAD_NAME);
+		updateHead(headName, newHead, upstreamCommit);
+		boolean stashConflicts = autoStashApply();
+		FileUtils.delete(rebaseState.getDir(), FileUtils.RECURSIVE);
+		if (stashConflicts)
+			return RebaseResult.STASH_APPLY_CONFLICTS_RESULT;
+		if (lastStepWasForward || newHead == null)
+			return RebaseResult.FAST_FORWARD_RESULT;
+		return RebaseResult.OK_RESULT;
+	}
+
+	private void checkSteps(List<RebaseTodoLine> steps)
+			throws InvalidRebaseStepException, IOException {
+		if (steps.isEmpty())
+			return;
+		if (RebaseTodoLine.Action.SQUASH.equals(steps.get(0).getAction())
+				|| RebaseTodoLine.Action.FIXUP.equals(steps.get(0).getAction())) {
+			if (!rebaseState.getFile(DONE).exists()
+					|| rebaseState.readFile(DONE).trim().length() == 0) {
+				throw new InvalidRebaseStepException(MessageFormat.format(
+						JGitText.get().cannotSquashFixupWithoutPreviousCommit,
+						steps.get(0).getAction().name()));
+			}
+		}
+
+	}
+
+	private RevCommit doSquashFixup(boolean isSquash, RevCommit commitToPick,
+			RebaseTodoLine nextStep, File messageFixup, File messageSquash)
+			throws IOException, GitAPIException {
+
+		if (!messageSquash.exists()) {
+			// init squash/fixup sequence
+			ObjectId headId = repo.resolve(Constants.HEAD);
+			RevCommit previousCommit = walk.parseCommit(headId);
+
+			initializeSquashFixupFile(MESSAGE_SQUASH,
+					previousCommit.getFullMessage());
+			if (!isSquash)
+				initializeSquashFixupFile(MESSAGE_FIXUP,
+					previousCommit.getFullMessage());
+		}
+		String currSquashMessage = rebaseState
+				.readFile(MESSAGE_SQUASH);
+
+		int count = parseSquashFixupSequenceCount(currSquashMessage) + 1;
+
+		String content = composeSquashMessage(isSquash,
+				commitToPick, currSquashMessage, count);
+		rebaseState.createFile(MESSAGE_SQUASH, content);
+		if (messageFixup.exists())
+			rebaseState.createFile(MESSAGE_FIXUP, content);
+
+		return squashIntoPrevious(
+				!messageFixup.exists(),
+				nextStep);
+	}
+
+	private void resetSoftToParent() throws IOException,
+			GitAPIException, CheckoutConflictException {
+		Ref orig_head = repo.getRef(Constants.ORIG_HEAD);
+		ObjectId orig_headId = orig_head.getObjectId();
+		try {
+			// we have already commited the cherry-picked commit.
+			// what we need is to have changes introduced by this
+			// commit to be on the index
+			// resetting is a workaround
+			Git.wrap(repo).reset().setMode(ResetType.SOFT)
+					.setRef("HEAD~1").call(); //$NON-NLS-1$
+		} finally {
+			// set ORIG_HEAD back to where we started because soft
+			// reset moved it
+			repo.writeOrigHead(orig_headId);
+		}
+	}
+
+	private RevCommit squashIntoPrevious(boolean sequenceContainsSquash,
+			RebaseTodoLine nextStep)
+			throws IOException, GitAPIException {
+		RevCommit newHead;
+		String commitMessage = rebaseState
+				.readFile(MESSAGE_SQUASH);
+
+		if (nextStep == null
+				|| ((nextStep.getAction() != Action.FIXUP) && (nextStep
+						.getAction() != Action.SQUASH))) {
+			// this is the last step in this sequence
+			if (sequenceContainsSquash) {
+				commitMessage = interactiveHandler
+						.modifyCommitMessage(commitMessage);
+			}
+			newHead = new Git(repo).commit()
+					.setMessage(stripCommentLines(commitMessage))
+					.setAmend(true).call();
+			rebaseState.getFile(MESSAGE_SQUASH).delete();
+			rebaseState.getFile(MESSAGE_FIXUP).delete();
+
+		} else {
+			// Next step is either Squash or Fixup
+			newHead = new Git(repo).commit()
+					.setMessage(commitMessage).setAmend(true)
+					.call();
+		}
+		return newHead;
+	}
+
+	private static String stripCommentLines(String commitMessage) {
+		StringBuilder result = new StringBuilder();
+		for (String line : commitMessage.split("\n")) { //$NON-NLS-1$
+			if (!line.trim().startsWith("#")) //$NON-NLS-1$
+				result.append(line).append("\n"); //$NON-NLS-1$
+		}
+		if (!commitMessage.endsWith("\n")) //$NON-NLS-1$
+			result.deleteCharAt(result.length() - 1);
+		return result.toString();
+	}
+
+	@SuppressWarnings("nls")
+	private static String composeSquashMessage(boolean isSquash,
+			RevCommit commitToPick, String currSquashMessage, int count) {
+		StringBuilder sb = new StringBuilder();
+		String ordinal = getOrdinal(count);
+		sb.setLength(0);
+		sb.append("# This is a combination of ").append(count)
+				.append(" commits.\n");
+		if (isSquash) {
+			sb.append("# This is the ").append(count).append(ordinal)
+					.append(" commit message:\n");
+			sb.append(commitToPick.getFullMessage());
+		} else {
+			sb.append("# The ").append(count).append(ordinal)
+					.append(" commit message will be skipped:\n# ");
+			sb.append(commitToPick.getFullMessage().replaceAll("([\n\r])",
+					"$1# "));
+		}
+		// Add the previous message without header (i.e first line)
+		sb.append("\n");
+		sb.append(currSquashMessage.substring(currSquashMessage.indexOf("\n") + 1));
+		return sb.toString();
+	}
+
+	private static String getOrdinal(int count) {
+		switch (count % 10) {
+		case 1:
+			return "st"; //$NON-NLS-1$
+		case 2:
+			return "nd"; //$NON-NLS-1$
+		case 3:
+			return "rd"; //$NON-NLS-1$
+		default:
+			return "th"; //$NON-NLS-1$
+		}
+	}
+
+	/**
+	 * Parse the count from squashed commit messages
+	 *
+	 * @param currSquashMessage
+	 *            the squashed commit message to be parsed
+	 * @return the count of squashed messages in the given string
+	 */
+	static int parseSquashFixupSequenceCount(String currSquashMessage) {
+		String regex = "This is a combination of (.*) commits"; //$NON-NLS-1$
+		String firstLine = currSquashMessage.substring(0,
+				currSquashMessage.indexOf("\n")); //$NON-NLS-1$
+		Pattern pattern = Pattern.compile(regex);
+		Matcher matcher = pattern.matcher(firstLine);
+		if (!matcher.find())
+			throw new IllegalArgumentException();
+		return Integer.parseInt(matcher.group(1));
+	}
+
+	private void initializeSquashFixupFile(String messageFile,
+			String fullMessage) throws IOException {
+		rebaseState
+				.createFile(
+						messageFile,
+						"# This is a combination of 1 commits.\n# The first commit's message is:\n" + fullMessage); //$NON-NLS-1$);
+	}
+
 	private String getOurCommitName() {
 		// If onto is different from upstream, this should say "onto", but
 		// RebaseCommand doesn't support a different "onto" at the moment.
@@ -423,7 +731,7 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 				List<String> fileList = dco.getToBeDeleted();
 				for (String filePath : fileList) {
 					File fileToDelete = new File(repo.getWorkTree(), filePath);
-					if (fileToDelete.exists())
+					if (repo.getFS().exists(fileToDelete))
 						FileUtils.delete(fileToDelete, FileUtils.RECURSIVE
 								| FileUtils.RETRY);
 				}
@@ -486,7 +794,8 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 		return parseAuthor(raw);
 	}
 
-	private RebaseResult stop(RevCommit commitToPick) throws IOException {
+	private RebaseResult stop(RevCommit commitToPick, RebaseResult.Status status)
+			throws IOException {
 		PersonIdent author = commitToPick.getAuthorIdent();
 		String authorScript = toAuthorScript(author);
 		rebaseState.createFile(AUTHOR_SCRIPT, authorScript);
@@ -504,7 +813,7 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 		// Remove cherry pick state file created by CherryPickCommand, it's not
 		// needed for rebase
 		repo.writeCherryPickHead(null);
-		return new RebaseResult(commitToPick);
+		return RebaseResult.result(status, commitToPick);
 	}
 
 	String toAuthorScript(PersonIdent author) {
@@ -541,67 +850,23 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 	private void popSteps(int numSteps) throws IOException {
 		if (numSteps == 0)
 			return;
-		List<String> todoLines = new ArrayList<String>();
-		List<String> poppedLines = new ArrayList<String>();
-		File todoFile = rebaseState.getFile(GIT_REBASE_TODO);
-		File doneFile = rebaseState.getFile(DONE);
-		BufferedReader br = new BufferedReader(new InputStreamReader(
-				new FileInputStream(todoFile), Constants.CHARACTER_ENCODING));
-		try {
-			// check if the line starts with a action tag (pick, skip...)
-			while (poppedLines.size() < numSteps) {
-				String popCandidate = br.readLine();
-				if (popCandidate == null)
-					break;
-				if (popCandidate.length() == 0)
-					continue;
-				if (popCandidate.charAt(0) == '#')
-					continue;
-				int spaceIndex = popCandidate.indexOf(' ');
-				boolean pop = false;
-				if (spaceIndex >= 0) {
-					String actionToken = popCandidate.substring(0, spaceIndex);
-					pop = Action.parse(actionToken) != null;
-				}
-				if (pop)
-					poppedLines.add(popCandidate);
-				else
-					todoLines.add(popCandidate);
-			}
-			String readLine = br.readLine();
-			while (readLine != null) {
-				todoLines.add(readLine);
-				readLine = br.readLine();
-			}
-		} finally {
-			br.close();
-		}
+		List<RebaseTodoLine> todoLines = new LinkedList<RebaseTodoLine>();
+		List<RebaseTodoLine> poppedLines = new LinkedList<RebaseTodoLine>();
 
-		BufferedWriter todoWriter = new BufferedWriter(new OutputStreamWriter(
-				new FileOutputStream(todoFile), Constants.CHARACTER_ENCODING));
-		try {
-			for (String writeLine : todoLines) {
-				todoWriter.write(writeLine);
-				todoWriter.newLine();
-			}
-		} finally {
-			todoWriter.close();
+		for (RebaseTodoLine line : repo.readRebaseTodo(
+				rebaseState.getPath(GIT_REBASE_TODO), true)) {
+			if (poppedLines.size() >= numSteps
+					|| RebaseTodoLine.Action.COMMENT.equals(line.getAction()))
+				todoLines.add(line);
+			else
+				poppedLines.add(line);
 		}
 
+		repo.writeRebaseTodoFile(rebaseState.getPath(GIT_REBASE_TODO),
+				todoLines, false);
 		if (poppedLines.size() > 0) {
-			// append here
-			BufferedWriter doneWriter = new BufferedWriter(
-					new OutputStreamWriter(
-							new FileOutputStream(doneFile, true),
-							Constants.CHARACTER_ENCODING));
-			try {
-				for (String writeLine : poppedLines) {
-					doneWriter.write(writeLine);
-					doneWriter.newLine();
-				}
-			} finally {
-				doneWriter.close();
-			}
+			repo.writeRebaseTodoFile(rebaseState.getPath(DONE), poppedLines,
+					true);
 		}
 	}
 
@@ -610,16 +875,9 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 		// we need to store everything into files so that we can implement
 		// --skip, --continue, and --abort
 
-		Ref head = repo.getRef(Constants.HEAD);
-		if (head == null || head.getObjectId() == null)
-			throw new RefNotFoundException(MessageFormat.format(
-					JGitText.get().refNotResolved, Constants.HEAD));
+		Ref head = getHead();
 
-		String headName;
-		if (head.isSymbolic())
-			headName = head.getTarget().getName();
-		else
-			headName = head.getObjectId().getName();
+		String headName = getHeadName(head);
 		ObjectId headId = head.getObjectId();
 		if (headId == null)
 			throw new RefNotFoundException(MessageFormat.format(
@@ -658,7 +916,7 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 
 		Collections.reverse(cherryPickList);
 		// create the folder for the meta information
-		FileUtils.mkdir(rebaseState.getDir());
+		FileUtils.mkdir(rebaseState.getDir(), true);
 
 		repo.writeOrigHead(headId);
 		rebaseState.createFile(REBASE_HEAD, headId.name());
@@ -666,28 +924,16 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 		rebaseState.createFile(ONTO, upstreamCommit.name());
 		rebaseState.createFile(ONTO_NAME, upstreamCommitName);
 		rebaseState.createFile(INTERACTIVE, ""); //$NON-NLS-1$
-		BufferedWriter fw = new BufferedWriter(new OutputStreamWriter(
-				new FileOutputStream(rebaseState.getFile(GIT_REBASE_TODO)),
-				Constants.CHARACTER_ENCODING));
-		fw.write("# Created by EGit: rebasing " + headId.name() + " onto "
-				+ upstreamCommit.name());
-		fw.newLine();
-		try {
-			StringBuilder sb = new StringBuilder();
-			ObjectReader reader = walk.getObjectReader();
-			for (RevCommit commit : cherryPickList) {
-				sb.setLength(0);
-				sb.append(Action.PICK.toToken());
-				sb.append(" "); //$NON-NLS-1$
-				sb.append(reader.abbreviate(commit).name());
-				sb.append(" "); //$NON-NLS-1$
-				sb.append(commit.getShortMessage());
-				fw.write(sb.toString());
-				fw.newLine();
-			}
-		} finally {
-			fw.close();
-		}
+
+		ArrayList<RebaseTodoLine> toDoSteps = new ArrayList<RebaseTodoLine>();
+		toDoSteps.add(new RebaseTodoLine("# Created by EGit: rebasing " + headId.name() //$NON-NLS-1$
+						+ " onto " + upstreamCommit.name())); //$NON-NLS-1$
+		ObjectReader reader = walk.getObjectReader();
+		for (RevCommit commit : cherryPickList)
+			toDoSteps.add(new RebaseTodoLine(Action.PICK, reader
+					.abbreviate(commit), commit.getShortMessage()));
+		repo.writeRebaseTodoFile(rebaseState.getPath(GIT_REBASE_TODO),
+				toDoSteps, false);
 
 		monitor.endTask();
 
@@ -706,6 +952,23 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 		return null;
 	}
 
+	private static String getHeadName(Ref head) {
+		String headName;
+		if (head.isSymbolic())
+			headName = head.getTarget().getName();
+		else
+			headName = head.getObjectId().getName();
+		return headName;
+	}
+
+	private Ref getHead() throws IOException, RefNotFoundException {
+		Ref head = repo.getRef(Constants.HEAD);
+		if (head == null || head.getObjectId() == null)
+			throw new RefNotFoundException(MessageFormat.format(
+					JGitText.get().refNotResolved, Constants.HEAD));
+		return head;
+	}
+
 	private boolean isInteractive() {
 		return interactiveHandler != null;
 	}
@@ -720,10 +983,7 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 	 */
 	public RevCommit tryFastForward(RevCommit newCommit) throws IOException,
 			GitAPIException {
-		Ref head = repo.getRef(Constants.HEAD);
-		if (head == null || head.getObjectId() == null)
-			throw new RefNotFoundException(MessageFormat.format(
-					JGitText.get().refNotResolved, Constants.HEAD));
+		Ref head = getHead();
 
 		ObjectId headId = head.getObjectId();
 		if (headId == null)
@@ -733,11 +993,7 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 		if (walk.isMergedInto(newCommit, headCommit))
 			return newCommit;
 
-		String headName;
-		if (head.isSymbolic())
-			headName = head.getTarget().getName();
-		else
-			headName = head.getObjectId().getName();
+		String headName = getHeadName(head);
 		return tryFastForward(headName, headCommit, newCommit);
 	}
 
@@ -782,6 +1038,12 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 	}
 
 	private void checkParameters() throws WrongRepositoryStateException {
+		if (this.operation == Operation.PROCESS_STEPS) {
+			if (rebaseState.getFile(DONE).exists())
+				throw new WrongRepositoryStateException(MessageFormat.format(
+						JGitText.get().wrongRepositoryState, repo
+								.getRepositoryState().name()));
+		}
 		if (this.operation != Operation.BEGIN) {
 			// these operations are only possible while in a rebasing state
 			switch (repo.getRepositoryState()) {
@@ -811,7 +1073,8 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 			}
 	}
 
-	private RebaseResult abort(RebaseResult result) throws IOException {
+	private RebaseResult abort(RebaseResult result) throws IOException,
+			GitAPIException {
 		try {
 			ObjectId origHead = repo.readOrigHead();
 			String commitId = origHead != null ? origHead.name() : null;
@@ -841,28 +1104,36 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 		}
 		try {
 			String headName = rebaseState.readFile(HEAD_NAME);
-			if (headName.startsWith(Constants.R_REFS)) {
 				monitor.beginTask(MessageFormat.format(
 						JGitText.get().resettingHead, headName),
 						ProgressMonitor.UNKNOWN);
 
+			Result res = null;
+			RefUpdate refUpdate = repo.updateRef(Constants.HEAD, false);
+			refUpdate.setRefLogMessage("rebase: aborting", false); //$NON-NLS-1$
+			if (headName.startsWith(Constants.R_REFS)) {
 				// update the HEAD
-				RefUpdate refUpdate = repo.updateRef(Constants.HEAD, false);
-				refUpdate.setRefLogMessage("rebase: aborting", false); //$NON-NLS-1$
-				Result res = refUpdate.link(headName);
-				switch (res) {
-				case FAST_FORWARD:
-				case FORCED:
-				case NO_CHANGE:
-					break;
-				default:
-					throw new JGitInternalException(
-							JGitText.get().abortingRebaseFailed);
-				}
+				res = refUpdate.link(headName);
+			} else {
+				refUpdate.setNewObjectId(repo.readOrigHead());
+				res = refUpdate.forceUpdate();
+
 			}
+			switch (res) {
+			case FAST_FORWARD:
+			case FORCED:
+			case NO_CHANGE:
+				break;
+			default:
+				throw new JGitInternalException(
+						JGitText.get().abortingRebaseFailed);
+			}
+			boolean stashConflicts = autoStashApply();
 			// cleanup the files
 			FileUtils.delete(rebaseState.getDir(), FileUtils.RECURSIVE);
 			repo.writeCherryPickHead(null);
+			if (stashConflicts)
+				return RebaseResult.STASH_APPLY_CONFLICTS_RESULT;
 			return result;
 
 		} finally {
@@ -907,57 +1178,6 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 		return true;
 	}
 
-	List<Step> loadSteps() throws IOException {
-		byte[] buf = IO.readFully(rebaseState.getFile(GIT_REBASE_TODO));
-		int ptr = 0;
-		int tokenBegin = 0;
-		ArrayList<Step> r = new ArrayList<Step>();
-		while (ptr < buf.length) {
-			tokenBegin = ptr;
-			ptr = RawParseUtils.nextLF(buf, ptr);
-			int nextSpace = RawParseUtils.next(buf, tokenBegin, ' ');
-			int tokenCount = 0;
-			Step current = null;
-			while (tokenCount < 3 && nextSpace < ptr) {
-				switch (tokenCount) {
-				case 0:
-					String actionToken = new String(buf, tokenBegin, nextSpace
-							- tokenBegin - 1);
-					tokenBegin = nextSpace;
-					if (actionToken.charAt(0) == '#') {
-						tokenCount = 3;
-						break;
-					}
-					Action action = Action.parse(actionToken);
-					if (action != null)
-						current = new Step(Action.parse(actionToken));
-					break;
-				case 1:
-					if (current == null)
-						break;
-					nextSpace = RawParseUtils.next(buf, tokenBegin, ' ');
-					String commitToken = new String(buf, tokenBegin, nextSpace
-							- tokenBegin - 1);
-					tokenBegin = nextSpace;
-					current.commit = AbbreviatedObjectId
-							.fromString(commitToken);
-					break;
-				case 2:
-					if (current == null)
-						break;
-					nextSpace = ptr;
-					int length = ptr - tokenBegin;
-					current.shortMessage = new byte[length];
-					System.arraycopy(buf, tokenBegin, current.shortMessage, 0,
-							length);
-					r.add(current);
-					break;
-				}
-				tokenCount++;
-			}
-		}
-		return r;
-	}
 
 	/**
 	 * @param upstream
@@ -1047,11 +1267,37 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 
 	/**
 	 * Enables interactive rebase
+	 * <p>
+	 * Does not stop after initialization of interactive rebase. This is
+	 * equivalent to
+	 * {@link RebaseCommand#runInteractively(InteractiveHandler, boolean)
+	 * runInteractively(handler, false)};
+	 * </p>
 	 *
 	 * @param handler
 	 * @return this
 	 */
 	public RebaseCommand runInteractively(InteractiveHandler handler) {
+		return runInteractively(handler, false);
+	}
+
+	/**
+	 * Enables interactive rebase
+	 * <p>
+	 * If stopAfterRebaseInteractiveInitialization is {@code true} the rebase
+	 * stops after initialization of interactive rebase returning
+	 * {@link RebaseResult#INTERACTIVE_PREPARED_RESULT}
+	 * </p>
+	 *
+	 * @param handler
+	 * @param stopAfterRebaseInteractiveInitialization
+	 *            if {@code true} the rebase stops after initialization
+	 * @return this instance
+	 * @since 3.2
+	 */
+	public RebaseCommand runInteractively(InteractiveHandler handler,
+			final boolean stopAfterRebaseInteractiveInitialization) {
+		this.stopAfterInitialization = stopAfterRebaseInteractiveInitialization;
 		this.interactiveHandler = handler;
 		return this;
 	}
@@ -1066,7 +1312,7 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 		 * @param steps
 		 *            initial configuration of rebase interactive
 		 */
-		void prepareSteps(List<Step> steps);
+		void prepareSteps(List<RebaseTodoLine> steps);
 
 		/**
 		 * Used for editing commit message on REWORD
@@ -1077,107 +1323,6 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 		String modifyCommitMessage(String commit);
 	}
 
-	/**
-	 * Describes rebase actions
-	 */
-	public static enum Action {
-		/** Use commit */
-		PICK("pick", "p"), //$NON-NLS-1$ //$NON-NLS-2$
-		/** Use commit, but edit the commit message */
-		REWORD("reword", "r"), //$NON-NLS-1$ //$NON-NLS-2$
-		/** Use commit, but stop for amending */
-		EDIT("edit", "e"); // later add SQUASH, FIXUP, etc. //$NON-NLS-1$ //$NON-NLS-2$
-
-		private final String token;
-
-		private final String shortToken;
-
-		private Action(String token, String shortToken) {
-			this.token = token;
-			this.shortToken = shortToken;
-		}
-
-		/**
-		 * @return full action token name
-		 */
-		public String toToken() {
-			return this.token;
-		}
-
-		@SuppressWarnings("nls")
-		@Override
-		public String toString() {
-			return "Action[" + token + "]";
-		}
-
-		static Action parse(String token) {
-			for (Action action : Action.values()) {
-				if (action.token.equals(token)
-						|| action.shortToken.equals(token))
-					return action;
-			}
-			throw new JGitInternalException(MessageFormat.format(
-					JGitText.get().unknownOrUnsupportedCommand, token,
-					Action.values()));
-		}
-	}
-
-	/**
-	 * Describes single rebase step
-	 */
-	public static class Step {
-		Action action;
-
-		AbbreviatedObjectId commit;
-
-		byte[] shortMessage;
-
-		Step(Action action) {
-			this.action = action;
-		}
-
-		/**
-		 * @return rebase action type
-		 */
-		public Action getAction() {
-			return action;
-		}
-
-		/**
-		 * @param action
-		 */
-		public void setAction(Action action) {
-			this.action = action;
-		}
-
-		/**
-		 * @return abbreviated commit SHA-1 of commit that action will be
-		 *         performed on
-		 */
-		public AbbreviatedObjectId getCommit() {
-			return commit;
-		}
-
-		/**
-		 * @return short message commit of commit that action will be performed
-		 *         on
-		 */
-		public byte[] getShortMessage() {
-			return shortMessage;
-		}
-
-		@SuppressWarnings("nls")
-		@Override
-		public String toString() {
-			return "Step["
-					+ action
-					+ ", "
-					+ ((commit == null) ? "null" : commit)
-					+ ", "
-					+ ((shortMessage == null) ? "null" : new String(
-							shortMessage)) + "]";
-		}
-	}
 
 	PersonIdent parseAuthor(byte[] raw) {
 		if (raw.length == 0)
@@ -1257,6 +1402,10 @@ public class RebaseCommand extends GitCommand<RebaseResult> {
 			return new File(getDir(), name);
 		}
 
+		public String getPath(String name) {
+			return (getDir().getName() + "/" + name); //$NON-NLS-1$
+		}
+
 		private static String readFile(File directory, String fileName)
 				throws IOException {
 			byte[] content = IO.readFully(new File(directory, fileName));
